#include "Router.h"

// constructor
Router::Router( sc_module_name name, const unsigned int target_num )
    : sc_module( name )
    //, i_socket( "i_socket" )
    , t_socket( "t_socket" )
    , targetNum( target_num )
{
    sc_assert( (0 < targetNum) && (targetNum < TARGET_MAX) );

    for (unsigned int i=0; i<targetNum; i++) {
        char socket_name[20];
        sprintf( socket_name, "i_socket_%d",  i );
        i_socket[i] = new tlm::tlm_initiator_socket<32>( socket_name );
        i_socket[i]->bind( *this );  // I/FȥХ
    }

    t_socket.bind( *this );  // I/FȥХ
}

Router::~Router()
{
    for (unsigned int i=0; i<targetNum; i++) {
        delete i_socket[i];
    }
}

unsigned int Router::decode_address( unsigned int address, unsigned int& masked_address )
{
    unsigned int target_nr =  (address >> 16) & 0xff; // target_nr is 0~255
    masked_address = address & 0xffff;
    return target_nr;
}

sc_dt::uint64 Router::compose_address( unsigned int target_nr, sc_dt::uint64 address)
{
    return (target_nr << 16) | (address & 0xffff);
}


// FW
void Router::b_transport( tlm::tlm_generic_payload &trans, sc_time &time ) 
{
    trans.set_response_status( tlm::TLM_ADDRESS_ERROR_RESPONSE  ); 
    unsigned int address = static_cast<unsigned int>(trans.get_address());
    unsigned int masked_address; // mask: 0x0000ffff
    unsigned int target_nr;

    target_nr = decode_address( address, masked_address );

    if (target_nr < targetNum) {
        trans.set_address( masked_address );
        (*i_socket[target_nr])->b_transport( trans, time );
    }
    else {
        cout << name() << " : ERROR, can not access Target No." << target_nr << endl;
        trans.set_response_status( tlm::TLM_ADDRESS_ERROR_RESPONSE  ); 
    }
}

// FW
tlm::tlm_sync_enum Router::nb_transport_fw(
        tlm::tlm_generic_payload &trans,
        tlm::tlm_phase &phase,
        sc_time &time )
{
    cout << "nb_transport_fw is not supoorted." << endl;
    trans.set_response_status( tlm::TLM_GENERIC_ERROR_RESPONSE );

    return tlm::TLM_COMPLETED;
}

// FW
unsigned int Router::transport_dbg( tlm::tlm_generic_payload &trans )
{
    //sc_dt::uint64 masked_address; 
    //unsigned int target_nr = decode_address( trans.get_address(), masked_address );
    //return (*i_socket[target_nr])->transport_dbg( trans );
    return 0;  // Debug not supported
}

bool Router::get_direct_mem_ptr( tlm::tlm_generic_payload &trans, tlm::tlm_dmi& dmi)
{
    //uint64 masked_address;
    //unsigned int target_nr = decode_address( trans.get_address(), masked_address );
    //bool status = (*i_socket[target_nr])->get_direct_mem_ptr( masked_address, dmi );
    //dmi.set_start_address( compose_address( target_nr, dmi.get_start_address() ) );
    //dmi.set_end_address( compose_address( target_nr, dmi.get_end_address() ) );
    //return status;
    return false; // DMI not supported
}


// BW 
tlm::tlm_sync_enum Router::nb_transport_bw( tlm::tlm_generic_payload& trans, tlm::tlm_phase &phase, sc_time& time )
{
    // not support
    trans.set_response_status( tlm::TLM_GENERIC_ERROR_RESPONSE );
    return tlm::TLM_COMPLETED;
}

// BW 
void Router::invalidate_direct_mem_ptr( sc_dt::uint64 a, sc_dt::uint64 b)
{
    // DMI unused
    // t_socket->invalidate_direct_mem_ptr( 0, (sc_dt::uint64)-1 );
}

